﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("MatchConditionOperator", "")]
  [StorableClass]
  public class MatchConditionOperator : SingleSuccessorOperator {
    public ILookupParameter<BoolValue> MatchConditionParameter {
      get { return (ILookupParameter<BoolValue>)Parameters["MatchCondition"]; }
    }
    public ILookupParameter<IClassifier> TargetMatchParameter {
      get { return (ILookupParameter<IClassifier>)Parameters["TargetMatchParameter"]; }
    }
    public ILookupParameter<IClassifier> MatchParameter {
      get { return (ILookupParameter<IClassifier>)Parameters["MatchParameter"]; }
    }

    [StorableConstructor]
    protected MatchConditionOperator(bool deserializing) : base(deserializing) { }
    protected MatchConditionOperator(MatchConditionOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public MatchConditionOperator()
      : base() {
      Parameters.Add(new LookupParameter<BoolValue>("MatchCondition", "True if the condition matches."));
      Parameters.Add(new ValueLookupParameter<IClassifier>("TargetMatchParameter", "Target that has to be matched by the match parameter."));
      Parameters.Add(new ValueLookupParameter<IClassifier>("MatchParameter", "The matching encoding contained in each sub-scope which is used for selection."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MatchConditionOperator(this, cloner);
    }

    public override IOperation Apply() {
      IClassifier target = TargetMatchParameter.ActualValue;
      IClassifier match = MatchParameter.ActualValue;
      MatchConditionParameter.ActualValue = new BoolValue(match.MatchCondition(target));
      return base.Apply();
    }
  }
}
